*! version 0.1.2 2013-08-21 | long freese | pw with at fix
* version 0.1.1 2013-08-21 | long freese | pre13, notes, sd-one, misc
* version 0.1.0 2013-08-19 | long freese | mchange2
//  discrete change using margins

capture program drop mchange2
capture program drop parse_amt
capture program drop parse_stat

program define mchange2, rclass

    version 11.2
    syntax [varlist(default=none)] [if] [in] , ///
        [                               ///
        at(string) /// added to end of at()
        AMount(string) /// AMOUNTBinary(string) ///
        ATMEANs /// use atmean option for margins
        brief ///
        DECimals(numlist >-1 integer) /// dec # in output
        delta(real 0) /// delta change instead of sd change
        DESCriptives /// descriptive statistics
        COMMANDs /// list commands
        DETails VERBose /// list margins output
        MATrix(string) /// name for saving r(table)
        OUTcome(string) /// user can specify outcomes TO BE ADDED
        replace /// replace matrix
        STATS(string asis) /// synonym for statistics
        STATistics(string asis) /// change ul ll z se p start end
        title(string) /// matrix title
        trim(numlist >-1 integer) /// percent to trim from min max
        UNCENtered /// uncentered change
        WIDth(numlist >5 integer) /// width in output
        NOIsily ///
        * ///
        ]

    tempname predvar prbasemat ATmat ATmatconstant descstats
    tempname blankmat varmat allchanges pwmatall prmat
    tempname prtvec tempvec prtmat amtmat pwmat

    local error = 0
    local pre13 = 0 // pre stata 13
    if (_caller()<13) local pre13 = 1

    if "`e(cmd)'"=="" | "`e(cmd)'"=="margins" {
        display as error "no estimation command is in memory"
        exit
    }
    estimates store _mc_model // last model estiamted
    local ATbase "`at'" // at specification

//  evaluate amount

    local AMTall "bin one sd rng marg" // all types in print order
    local AMTabbrev "bin one std rng mar" // all types in print order
    local AMTdc  "bin one sd rng" // all dc types
    local AMTfac "bin" // default factor vars
    local AMTvar "one sd marg" // default for continuous
    foreach amt in `AMTall' {
        local is`amt' = 0
    }

    * verify amounts()
    local amount : subinstr local amount "delta" "sd", all
    if ("`amount'"=="") local amount "`AMTvar'"
    if ("`amount'"=="all") local amount "`AMTall'"
    local isbad = 0
    if "`amount'"!="" { // amounts for continuous variables
        local new ""
        foreach anm in `amount' {
            parse_amt `anm'
            local new "`new'`s(amount)' "
            if "`s(isbad)'"=="1" {
                display as error "invalid amount specified: `anm'"
                local isbad = 1
                exit
            }
            else local is`s(amount)' = 1
        }
        local amount "`new'"
    }

    if ("`amount'"!="") local AMTvar "`amount'" // override default
    local AMTvardc : subinstr local amount "marg" "", all

//  evaluate statistics

    local STATall       "change  pvalue ll ul   z    se  start end"
    local STATabbrev    "chg     pva    cil ciu zva  ser str   end"
    local STATnostrtend "change pvalue ll ul z se"
    local STATdefault   "change pvalue"

    if ("`stats'"!="" & "`statistics'"=="") local statistics "`stats'"
    if ("`statistics'"=="ci") local statistics "change ll ul"
    if ("`statistics'"=="all") local statistics `STATall'

    if "`statistics'"!="" { // select statistics
        local new ""
        foreach snm in `statistics' {
            parse_stat `snm'
            local new "`new'`s(stat)' "
            if "`s(isbad)'"=="1" {
                display as error "invalid statistic specified: `s(stat)'"
                local isbad = 1
                exit
            }
        }
        local statistics "`new'"
    }
    if ("`isbad'"=="1") exit

    local STATprt "`statistics'"
    if ("`STATprt'"=="") local STATprt "`STATdefault'"
    if ("`STATprt'"=="all") local STATprt "`STATall'"
    local iposu : list posof "`ul'" in STATprt
    local iposl : list posof "`ll'" in STATprt
    local isci = 0
    if (`iposu'>0 | `iposl'>0) isci = 1

//  model information

    _rm_modelinfo2
    local varnms `s(rhs_core)'
    if ("`varlist'"!="") local varnms "`varlist'"
    local nvarnms = wordcount("`varnms'")
    local TYPEfac `s(rhs_typefactor)' // which vars are factor vars
    local TYPEvar `s(rhs_typevariable)' // which are c. variables

    local MDLtitle "`r(cmdnm)'"
    local is_svy = `r(cmdsvy)'
    if (`is_svy') local MDLtitle "svy `MDLtitle'"
    local MDLbrm = `r(cmdbin)'
    local lhsncats = `r(lhscatn)'
    local lhsnm "`r(lhsnm)'"
    local lhscatnms "`r(lhscatnms)'"
    local MDLcrm = `r(cmdcrm)' // count model
    if `MDLcrm' {
        local lhsncats = 1
        local lhscatnms "Rate"
    }
    local lhscatvals "`r(lhscatvals)'"
    if (`MDLbrm' | `MDLcrm') local lhscatvals "1"

    local singlecolnum = 0
    if (`lhsncats'==1) local singlecolnum = 1
    if `MDLbrm'==1 local singlecolnum = 2 // brm in col 2 of matrices

//  decode options & create footnotes

    if ("`decimals'"=="") local decimals = 3
    if ("`width'"=="") local width = 9
    local twid = 12 // width of left margins

    local quicmd "quietly" // listing of command
    if ("`commands'"=="commands") local quicmd "noisily"
    if ("`verbose'"=="verbose") local details "details"
    local quimar "quietly"
    if "`details'" == "details" {
        local quimar "noisily"
        local quicmd "noisily"
    }

    local NOTEsample ""
    if ("`if'"=="") local if "if e(sample)==1"
    else local if "`if' & e(sample)==1"
    if ("`if'"!="") & ("`if'"!="if e(sample)==1") {
        local NOTEsample " Sample selection: `if'"
    }
    if "`in'" !="" {
        if ("`NOTEsample'"=="") local NOTEsample " Sample selection `in'"
        else local NOTEsample "`NOTEsample' `in'"
    }

    local NOTEtrim ""
    if ("`trim'"=="") local trim = 0
    if (`trim'!=0) local NOTEtrim "Range trimmed by `trim' percent."

    local NOTEdelta ""
    if (`delta'!=0) local NOTEdelta "Delta equals `delta'."

//  labels

    if ("`uncentered'"=="uncentered") local LBLcntr "unctr"
    else local LBLcntr "cntr"
    if ("`uncentered'"=="uncentered") local LBLcntrabbrev "unc"
    else local LBLcntrabbrev "ctr"
    if (`delta'!=0) local LBLstddev "delta"
    else local LBLstddev "SD"
    local LBLbin "0_to_1"
    local LBLone "+1_`LBLcntr'"
    local LBLsd "+`LBLstddev'_`LBLcntr'"
    local LBLmarg "Marginal"
    local trimto = 100 - `trim'
    if (`trim'==0) local LBLrng "Range"
    else local LBLrng "`trim'%_to_`trimto'%"

    local LBLchange `"Change"'
    local LBLul `"UL"'
    local LBLll `"LL"'
    local LBLz `"z-value"'
    local LBLse `"Std. Err."'
    local LBLpvalue `"P>|z|"'
    local LBLstart `"From"'
    local LBLend `"To"'

    * location in collection matrix
    local ROWchange = 1
    local ROWse = 2
    local ROWz = 3
    local ROWpvalue = 4
    local ROWll = 5
    local ROWul = 6
    local ROWstart = 7
    local ROWend = 8
    local nstattypes = 8

//  margins to get labels & r(at)

    qui sum `e(depvar)'
    capture predict `predvar', outcome(`r(min)')
    capture drop `predvar'
    if (_rc==0) local optpred "predict(outcome(`r(min)'))"
    * local options holds passthrough options for margins
    qui margins `if' `in', at(`ATbase') `atmeans' `options' `optpred'
    local nobsifin = r(N)
    local pr1base = el(r(b),1,1) // used for base predictions
    local lvlis = r(level)
    local NOTEci ""
    if `isci' local NOTEci "`lvlis' percent confidence interval."

    capture matrix list r(at)
    if _rc==0 { // r(at) exists
        if rowsof(r(at))>1 {
            display as error ///
"only one value of variable allowed in at(); e.g., at(male=(0 1)) is invalid"
            exit
        }
        quietly mlistat, saveconstant(`ATmatconstant')
        local ATmatconstantis = `s(isconstant)' // at matrix fixed values
        if `ATmatconstantis'==1 {
            matrix rownames `ATmatconstant' = "at"
            return matrix atconstant = `ATmatconstant', copy
        }
    }
    matrix `ATmat' = r(at)
    local ATnms : colnames `ATmat'
    _rm_marginsnames // names used by margins for predictions
    local Mmath_noequal `s(math_noequal)' // used

// predictions at baseline values

    capture drop matrix `prbasemat'
    local basecatnames ""
    if ("`atmeans'"=="atmeans") local baseheader "Predictions at base values"
    else local baseheader "Average predictions"

    if `MDLbrm' {
        local pr0base = 1 - `pr1base'
        matrix `prbasemat' = `pr0base', `pr1base'
        matrix rownames `prbasemat' = "Pr(y|base)"
        matrix colnames `prbasemat' = `lhscatnms'
    }
    else if `MDLcrm' {
        if "`atmeans'"=="atmeans" local baseheader "Prediction at base values"
        else local baseheader "Average prediction"
        matrix `prbasemat' = `pr1base'
        matrix rownames `prbasemat' = "Rate"
        local basecatnames "nonames"
    }
    else {
        foreach catval in `lhscatvals' { // outcome value
            qui margins `if' `in', at(`ATbase') `atmeans' `options' ///
                predict(outcome(`catval'))
            local pval = el(r(b),1,1)
            matrix `prbasemat' = nullmat(`prbasemat') , `pval'
        }
        matrix rownames `prbasemat' = `""Pr(y|base)""'
        matrix colnames `prbasemat' = `lhscatnms'
    }

//  compute effects

    * template for results matrix for 1 variable or 1 contrast
    matrix `blankmat' = J(8,`lhsncats',.z) // 8 stats by # outcomes
    matrix colna `blankmat' = `lhscatnms'
    matrix rowna `blankmat' = change se z pvalue ll ul start end

    foreach varnm in `varnms' { // selected variables

        capture matrix drop `varmat'

        local ipos : list posof "`varnm'" in TYPEfac
        if (`ipos'>0) local type fac
        else local type var

        // compute from and to values for discrete changes

        if "`type'" == "var" {

            // descriptives

            if `issd' | `isone' | `isrng' {
                qui sum `varnm' `if' `in'
                local mn = r(mean)
                local sd = r(sd)
                local min = r(min)
                local max = r(max)
                if `is_svy' {
                    capture svy : mean  `varnm' `if' `in'
                    if _rc!= 0 {
                        display as error ///
             "cannot compute std dev for factor variables with svy data"
                        exit
                        continue
                    }
                    qui estat sd
                    local mn = el(r(mean),1,1)
                    local sd = el(r(sd),1,1)
                }
                if (`delta'!=0) local sd = `delta'
                local sd2 = `sd'/2
            } // get desc

            // atspecs for all amounts of changes

            * at value needed for one sd amounts
            local ipos : list posof "`varnm'" in ATnms
            local ATval = `ATmat'[1,`ipos']
            if (!missing(`ATval')) local ATvarisset = 1
            else local ATvarisset = 0

            * note: start value at start; end value at end. If ATbase
            * includes value of varnm, the first and last predictions will
            * be correct even if 3rd prediction is in middel

            local ATbin "at(`varnm'=0 `ATbase' `varnm'=1)"

            if `ATvarisset'==1 {
                if "`uncentered'"=="" {
                    local F1 = `ATval' - .5
                    local T1 = `ATval' + .5
                    local Fsd = `ATval' - `sd2'
                    local Tsd = `ATval' + `sd2'
                }
                else { // uncentered
                    local F1 = `ATval'
                    local T1 = `ATval' + 1
                    local Fsd = `ATval'
                    local Tsd = `ATval' + `sd'
                }
                local ATone "at(`varnm'=`F1' `ATbase' `varnm'=`T1')"
                local ATsd "at(`varnm'=`Fsd' `ATbase' `varnm'=`Tsd')"
            }

            if `ATvarisset'==0 {
                if "`uncentered'"=="" {
                    local vF1 "`varnm'=gen(`varnm'-.5)"
                    local vT1 "`varnm'=gen(`varnm'+.5)"
                    local vFsd "`varnm'=gen(`varnm'-`sd2')"
                    local vTsd "`varnm'=gen(`varnm'+`sd2')"
                }
                else { // uncentered
                    local vF1 "`varnm'=gen(`varnm')"
                    local vT1 "`varnm'=gen(`varnm'+1)"
                    local vFsd "`varnm'=gen(`varnm')"
                    local vTsd "`varnm'=gen(`varnm'+`sd')"
                }
                local ATone "at(`vF1' `ATbase') at(`ATbase' `vT1')"
                local ATsd "at(`vFsd' `ATbase') at(`ATbase' `vTsd')"
                if `pre13' {
                    local ATone "pre13"
                    local ATsd "pre13"
                }
            }

            if `trim'!=0 {
                local trimUB = 100 - `trim'
                qui centile `varnm' `if' `in', centile(`trim' `trimUB')
                local F = r(c_1)
                local T = r(c_2)
            }
            else {
                qui sum `varnm' `if' `in'
                local F = r(min)
                local T = r(max)
            }
            local ATrng "at(`varnm'=`F' `ATbase' `varnm'=`T')"

            // compute predictions

            foreach amt in `AMTvar' {

                matrix `amtmat' = `blankmat'
                * build row label=> varnm:A<amt>C<ctr>S<stat>
                local ipos : list posof "`amt'" in AMTall
                local ais : word `ipos' of `AMTabbrev'
                local rn "`varnm':A`ais'C`LBLcntrabbrev'"
                matrix rowna `amtmat' = `rn'Sch `rn'Sse `rn'Szv `rn'Spv ///
                    `rn'Sll `rn'Sul `rn'Sst `rn'Sen

                if "`amt'"=="marg" {

                    local colnum = 0 // outcome column
                    foreach catval in `lhscatvals' { // outcome value
                        local optpred ""
                        if (`MDLbrm') local colnum = 1 // pr(1) in col 1+1
                        else if (`MDLcrm') local colnum = 0 // mu is col 0+1
                        else local optpred "predict(outcome(`catval'))"
                        local ++colnum
                        local cmdis "margins `if' `in', dydx(`varnm')"
                        local cmdis "`cmdis' at(`ATbase') `atmeans'"
                        local cmdis "`cmdis' `optpred' `options'"
                        _rm_cleancmd `cmdis'
                        local cmdis "`s(cmd)'"
                        `quicmd' display "Command: `cmdis'"
                        `quimar' `cmdis'
                        mat `amtmat'[1,`colnum'] = el(r(table),1,1) // b
                        mat `amtmat'[2,`colnum'] = el(r(table),2,1) // se
                        mat `amtmat'[3,`colnum'] = el(r(table),3,1) // z
                        mat `amtmat'[4,`colnum'] = el(r(table),4,1) // p
                        mat `amtmat'[5,`colnum'] = el(r(table),5,1) // ll
                        mat `amtmat'[6,`colnum'] = el(r(table),6,1) // ul
                    } // outcome

                    matrix `varmat' = nullmat(`varmat') \ `amtmat'

                } // marg

                else { // dc

                    local colnum = 0 // outcome column
                    foreach catval in `lhscatvals' {
                        local optpred ""
                        if (`MDLbrm') local colnum = 1
                        else if (`MDLcrm') local colnum = 0
                        else local optpred "predict(outcome(`catval'))"
                        local ++colnum
                        local cmdis "margins `if' `in', `AT`amt'' post"
                        local cmdis "`cmdis' `atmeans' `optpred' `options'"
                        _rm_cleancmd `cmdis'
                        local cmdis "`s(cmd)'"
                        if "`AT`amt''"=="pre13" {
                            local NOTEpre13 ///
                            ".g indicates effects requires Stata 13 or later."
                            mat `amtmat'[1,`colnum'] = J(8,1,.g)
                        }
                        else {
                            `quicmd' display "Command: `cmdis'"
                            `quimar' `cmdis'
                            local nat = colsof(r(b))
                            local locT = `nat'
                            `quicmd' display "Command: mlincom `locT' - 1"
                            `quimar' mlincom `locT' - 1
                            mat `amtmat'[1,`colnum'] = r(est) // b
                            mat `amtmat'[2,`colnum'] = r(se) // se
                            mat `amtmat'[3,`colnum'] = r(z) // z
                            mat `amtmat'[4,`colnum'] = r(p) // p
                            mat `amtmat'[5,`colnum'] = r(lb) // ll
                            mat `amtmat'[6,`colnum'] = r(ub) // ul
                            mat `amtmat'[7,`colnum'] = el(e(b),1,1) // From
                            mat `amtmat'[8,`colnum'] = el(e(b),1,`locT') // To
                            qui estimates restore _mc_model
                        }
                    } // outcome

                    matrix `varmat' = nullmat(`varmat') \ `amtmat'

                } // dc

            } // amt

            matrix `allchanges' = nullmat(`allchanges') \ `varmat'

        } // type var

        // factor variables

        else if "`type'" == "fac" {

            qui tab `varnm' `if' `in'
            local ncat = r(r)
            local tmpnpw = ((`ncat'*`ncat')-`ncat')/2
            forvalues i = 1/`tmpnpw' {
                tempname pwmat`i'
            }
            local colnum = 0 // outcome column
            foreach catval in `lhscatvals' { // outcome value

                local optpred ""
                if (`MDLbrm') local colnum = 1
                else if (`MDLcrm') local colnum = 0
                else local optpred "predict(outcome(`catval'))"
                local ++colnum

                local cmdis "margins `varnm' `if' `in', at(`ATbase')"
                local cmdis "`cmdis' `atmeans' `optpred' `options' pwcompare"
                _rm_cleancmd `cmdis'
                local cmdis "`s(cmd)'"
                `quicmd' display "Command: `cmdis'"
                `quimar' `cmdis'

                matrix `pwmatall' = r(table_vs) // pw contrasts
                local npw = colsof(`pwmatall')
                local npw`varnm' = `npw'
                local pwcolnms : colnames(`pwmatall')

                matrix `prmat' = r(table) // outcome probabiliteis
                local npr = colsof(`prmat')
                local prcolnms : colnames(`prmat')
                * remove all but numbers from column names
                local prcolnums : subinstr local prcolnms ".`varnm'" "", all
                local prcolnums : subinstr local prcolnums "bn" "", all

                qui `noisily' matlist `prmat', ///
                    title(pr rtable) format(%10.4f)
                qui `noisily' matlist `pwmatall', ///
                    title(pw rtable_vs) format(%10.4f)

                forvalues ipw = 1/`npw' {

                    * pw names using value labels
                    _ms_element_info, element(`ipw') matrix(`pwmatall') compare
                    local pwnm `"`r(level)'"'
                    local pwnmlen = length(`"`r(level)'"') // for matlist
                    if (`pwnmlen'>=`twid') local twid = `pwnmlen' + 1

                    * get category #'s for contrasts
                    local pwcolnm : word `ipw' of `pwcolnms'
                    local pwcolnm : subinstr local pwcolnm ".`varnm'" "", all
                    local pwcolnm : subinstr local pwcolnm "vs" " ", all
                    local pwcolnm : subinstr local pwcolnm "bn" "", all

                    * locations in prmat for start & end predicitons
                    local pwstval : word 2 of `pwcolnm'
                    local pwstcol : list posof "`pwstval'" in prcolnums
                    local pwenval : word 1 of `pwcolnm'
                    local pwencol : list posof "`pwenval'" in prcolnums

                    * matrix for pw comparison #ipw
                    capture confirm matrix `pwmat`ipw''
                    if _rc!=0 { // doesn't exist so create
                        * rowname <varnm>!<pwnm>:C<center>A<amount>S<stat>
                        local rn : subinstr local pwnm " " "_", all
                        local rn "`varnm'!`rn':C`LBLcntrabbrev'Abi"
                        mat `pwmat`ipw'' = `blankmat'
                        matrix rowna `pwmat`ipw'' = `rn'Sch `rn'Sse `rn'Szv ///
                            `rn'Spv `rn'Sll `rn'Sul `rn'Sst `rn'Sen
                    }

                    mat `pwmat`ipw''[7,`colnum'] = `prmat'[1,`pwstcol']
                    mat `pwmat`ipw''[8,`colnum'] = `prmat'[1,`pwencol']
                    mat `pwmat`ipw''[1,`colnum'] = `pwmatall'[1,`ipw']
                    mat `pwmat`ipw''[2,`colnum'] = `pwmatall'[2,`ipw']
                    mat `pwmat`ipw''[3,`colnum'] = `pwmatall'[3,`ipw']
                    mat `pwmat`ipw''[4,`colnum'] = `pwmatall'[4,`ipw']
                    mat `pwmat`ipw''[5,`colnum'] = `pwmatall'[5,`ipw']
                    mat `pwmat`ipw''[6,`colnum'] = `pwmatall'[6,`ipw']

                } // ipw loop

            } // outcome

            forvalues ipw = 1/`npw' { // results for all contrasts
                matrix `allchanges' = nullmat(`allchanges') \ `pwmat`ipw''
            }

        } // type fac

    } // varnm

    return matrix changes = `allchanges', copy
    qui `noisily' matlist `allchanges', title(_mc_allchanges)

//  select results for printing

    capture matrix drop `prtmat'

    //  combine results into table

    if `singlecolnum'>0 { // binary or count

        * columns are statistics
        local statcolnms ""
        foreach stat in `STATprt' {
            local statcolnms `"`statcolnms'`""`LBL`stat''""' "'
        }

        local rowst = -`nstattypes' // start location in `allchanges'
        foreach varnm in `varnms' {

            local ipos : list posof "`varnm'" in TYPEfac
            if (`ipos'>0) local type fac
            else local type var

            if "`type'"=="var" { // get dc and marg

                foreach amt in `AMTvar' {
                    local rowst = `rowst' + 8
                    local rownm "`varnm':`LBL`amt''"
                    capture matrix drop `prtvec'
                    foreach snm in `STATprt' {
                        local irow = `rowst' + `ROW`snm''
                        local stat = `allchanges'[`irow',`singlecolnum']
                        matrix `prtvec' = nullmat(`prtvec') , `stat'
                    }
                    matrix rowna `prtvec' = `rownm'
                    matrix colna `prtvec' = `statcolnms' // `"`statcolnms'"'
                    matrix `prtmat' = nullmat(`prtmat') \ `prtvec'
                } // amt

            } // type var

            else { // type fac get pw comparisons

                forvalues ipw = 1/`npw`varnm'' { // # of pw comparisons

                    local rowst = `rowst' + `nstattypes'
                    capture matrix drop `prtvec'
                    foreach stat in `STATprt' {
                        local irow = `rowst' + `ROW`stat''
                        mat `tempvec' = `allchanges'[`irow',1...]
                        local stat = `allchanges'[`irow',`singlecolnum']
                        matrix `prtvec' = nullmat(`prtvec') , `stat'
                    }

                    * split varnm & contrast name
                    local pweqnm : roweq `tempvec' // varnm!contrastnm
                    local pwnmst = strpos("`pweqnm'","!") + 1
                    local pwnm = substr("`pweqnm'",`pwnmst',.)
                    local rownm "`varnm':`pwnm'"
                    matrix rowna `prtvec' = `rownm'
                    matrix colna `prtvec' = `statcolnms' // `"`statcolnms'"'
                    matrix `prtmat' = nullmat(`prtmat') \ `prtvec'

                } // pw comparison
            } // type fac
        } // varnm
    } // single column of output

    // multiple outcomes

    else {

        local prechgnms "" // names before change statistic
        local postchgnms "" // names after
        local ischgstat = 0
        foreach stat in `STATprt' {
            if "`stat'"=="change" local ischgstat = 1
            else if `ischgstat'==0 {
                local prechgnms `"`prechgnms'`""`LBL`stat''""' "'
            }
            else local postchgnms `"`postchgnms'`""`LBL`stat''""' "'
        }

        local rowst = -`nstattypes' // start location in `allchanges'
        foreach varnm in `varnms' {

            local ipos : list posof "`varnm'" in TYPEfac
            if (`ipos'>0) local type fac
            else local type var

            capture matrix drop `varmat'

            if "`type'"=="var" { // get dc and marg
                foreach amt in `AMTvar' {
                    local rowst = `rowst' + `nstattypes'
                    capture matrix drop `amtmat'
                    foreach snm in `STATprt' {
                        local irow = `rowst' + `ROW`snm''
                        matrix `prtvec' = `allchanges'[`irow',1...]
                        matrix `amtmat' = nullmat(`amtmat') \ `prtvec'
                    }
                    matrix roweq `amtmat' = `varnm'
                    matrix rowna `amtmat' = `prechgnms' `"`LBL`amt''"' `postchgnms'
                    matrix `varmat' = nullmat(`varmat') \ `amtmat'
                } // amt
                qui `noisily' matlist `varmat', format(%9.3f) title(varmat `varnm')
                matrix `prtmat' = nullmat(`prtmat') \ `varmat'
            } // type var

            else { // type fac combined pw comparisons
                forvalues ipw = 1/`npw`varnm'' { // # comparison
                    local rowst = `rowst' + `nstattypes'
                    capture matrix drop `pwmatall'
                    foreach snm in `STATprt' {
                        local irow = `rowst' + `ROW`snm''
                        matrix `prtvec' = `allchanges'[`irow',1...]
                        matrix `pwmatall' = nullmat(`pwmatall') \ `prtvec'
                    }
                    matrix roweq `pwmatall' = `varnm'
                    local pweqnm : roweq `prtvec' // varnm!contrastnm
                    local pwnmst = strpos("`pweqnm'","!") + 1
                    local pwnm = substr("`pweqnm'",`pwnmst',.)
                    matrix rowna `pwmatall' = `prechgnms' `pwnm' `postchgnms' // `"`LBL`amt''"'
                    matrix `varmat' = nullmat(`varmat') \ `pwmatall'
                } // pw contrast
                matrix `prtmat' = nullmat(`prtmat') \ `varmat'
                qui `noisily' matlist `varmat', format(%9.3f) title(varmat `varnm')

            } // type fac

        } // varnm

    } // multiple categories

//  print results

    if ("`title'"=="") local titlenm "`MDLtitle': Changes in `Mmath_noequal'"
    else local titlenm "`title'"
    display
    display as result "`titlenm' | Number of obs = `nobsifin'" _continue
    display


    //  marginal effects

    matlist `prtmat', format(%`width'.`decimals'f) lines(oneline) ///
        underscore twidth(`twid') // title("{bf}`Slbl_`s''{sf}")
    return matrix table = `prtmat'

    //  base predictions

    if "`brief'"=="" {
        matlist `prbasemat', noheader format(%`width'.`decimals'f) ///
            `basecatnames' title(`baseheader')
    }
    return matrix basepred = `prbasemat'

    //  at values

    if "`ATmatconstantis'"=="1" & "`brief'"=="" {
        matlist `ATmatconstant', format(%`width'.`decimals'g) ///
            title(Base values of regressors)
    }

    //  descriptives

    if "`descriptives'"=="descriptives" {
        qui estat summarize
        matrix `descstats' = r(stats)
        matrix colnames `descstats' = Mean `""Std. Dev.""' Min Max
        display
        display "Descriptive statistics for estimation sample"
        display
        local fmt "%width'.`decimals'f"
        _matrix_table `stats', format(`fmt' `fmt' `fmt' `fmt')
        return matrix stats = `descstats'
    }

    //  notes

    if  "`brief'"=="" {
        display
        local i = 1
        if "`atmeans'"=="atmeans" {
            display _col(1) "`i': Estimates with margins option atmeans."
            local ++i
        }
        if "`NOTEsample'"!="" {
            display _col(1) "`i':`NOTEsample'"
            local ++i
        }
        if "`NOTEdelta'"!="" {
            display _col(1) "`i': `NOTEdelta'"
            local ++i
        }
        if "`NOTEci'"!="" {
            display _col(1) "`i': `NOTEci'"
            local ++i
        }
        if "`NOTEpre13'"!="" {
            display _col(1) "`i': `NOTEpre13'"
            local ++i
        }
    } // brief

//  export for orplot and meplot
/*
//  matrices for orplot and dcplot

    if `is_mainvars' {
        matrix _mchange_plot_dc = `_all_change' // for orplot and dcplot
        matrix _mchange_plot_pvalue = `_all_pvalue' // for orplot & dcplot
    }

    capture matrix drop _mchange_plotPW_pvalue
    capture matrix drop _mchange_plotPW_dc
    local Vval = 0
    foreach VnmPW in `vars_PW' {
        local ++Vval
        forvalues i = 1/`nPW_`VnmPW'' {
            matrix _mchange_plotPW_dc ///
                = nullmat(_mchange_plotPW_dc) \ _PWdc_V`Vval'PW`i'
            matrix _mchange_plotPW_pvalue ///
                = nullmat(_mchange_plotPW_pvalue) \ _PWpvalue_V`Vval'PW`i'
            capture matrix drop _PWpvalue_V`Vval'PW`i'
            capture matrix drop _PWdc_V`Vval'PW`i'
        }
    }
*/

end

* decode amount
program define parse_amt, sclass
    local isbad = 1
    local amt "`1'"
    local is = inlist("`amt'","r","ra","ran","rang","range","rng")
    if `is'==1 {
        local amt "rng"
        local isbad = 0
    }
    local is = inlist("`amt'","b","bi","bin","bina","binar","binary")
    if `is'==1 {
        local amt "bin"
        local isbad = 0
    }
    local is = inlist("`amt'","1","o","on","one")
    if `is'==1 {
        local amt "one"
        local isbad = 0
    }
    local is = inlist("`amt'","s","sd")
    if `is'==1 {
        local amt "sd"
        local isbad = 0
    }
    local is = inlist("`amt'","m","ma","mar","marg","margi","margin")
    if `is'==1 {
        local amt "marg"
        local isbad = 0
    }
    else {
        local is = inlist("`amt'","margina","marginal","dydx","partial")
        if `is'==1 {
            local amt "marg"
            local isbad = 0
        }
    }
    sreturn local amount "`amt'"
    sreturn local isbad "`isbad'"
end

program define parse_stat, sclass
    local isbad = 1
    local stat "`1'"
    local is = inlist("`stat'","z","zv","zva","zval","zvalu","zvalue")
    if `is'==1 {
        local stat "z"
        local isbad = 0
    }
    local is = inlist("`stat'","t","tv","tva","tval","tvalu","tvalue")
    if `is'==1 {
        local stat "z"
        local isbad = 0
    }
    local is = inlist("`stat'","from")
    if `is'==1 {
        local stat "start"
        local isbad = 0
    }
    local is = inlist("`stat'","st","sta","star","start")
    if `is'==1 {
        local stat "start"
        local isbad = 0
    }
    local is = inlist("`stat'","to")
    if `is'==1 {
        local stat "end"
        local isbad = 0
    }
    local is = inlist("`stat'","en","end")
    if `is'==1 {
        local stat "end"
        local isbad = 0
    }
    local is = inlist("`stat'","change","first","dydx","estimat","estimate")
    if `is'==1 {
        local stat "change"
        local isbad = 0
    }
    local is = inlist("`stat'","dc","e","es","est","esti","estim","estima")
    if `is'==1 {
        local stat "change"
        local isbad = 0
    }
    local is = inlist("`stat'","upper","ub","ul")
    if `is'==1 {
        local stat "ul"
        local isbad = 0
    }
    local is = inlist("`stat'","lower","lb","ll")
    if `is'==1 {
        local stat "ll"
        local isbad = 0
    }
    local is = inlist("`stat'","p","pv","pva","pval","pvalu","pvalue")
    if `is'==1 {
        local stat "pvalue"
        local isbad = 0
    }
    local is = inlist("`stat'","se","s")
    if `is'==1 {
        local stat "se"
        local isbad = 0
    }
    sreturn local stat "`stat'"
    sreturn local isbad "`isbad'"

end

exit

DOCUMENTATON

Globals
    global mchange_singleoutcome = `singleoutcome'
    global mchange_atspec

    r(basepred)
    r(changes)
    r(atconstant)
    r(stats)

    CRM only for rate

Document
    Options
    Returns
    Matrices

NOTES
/* not needed
        if `isrng'==1 {
            display _col(1) "`i': `NOTEtrim'"
            local ++i
        }

        if "`NOTEasobs'"!=="" {
            display _col(1) "`i': `NOTEasobs'"
            local ++i
        }
*/

